/** Copyright 2012
*
* Author: Ismael PONCE GORDILLO
*
* Licensed under the EUPL, Version 1.1 or – as soon they will be
* approved by the European Commission – subsequent versions of the
* EUPL (the "Licence"); you may not use this work except in
* compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* https://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the Licence is distributed on an "AS
* IS" BASIS, * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
* express or implied.
* See the Licence for the specific language governing permissions
* and limitations under the Licence.
*/
import java.io.File;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Locale;
import java.util.ResourceBundle;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.TabItem;

public class PCuaderno {
	final String espanol = "es";
	final Locale ingles = new Locale("en","UK");
	final String arch_mensajes="messages";
	public ResourceBundle mensajes = ResourceBundle.getBundle(arch_mensajes,(Locale.getDefault().getLanguage()==espanol)?Locale.getDefault():ingles);  
	public static PCuaderno MiPCuaderno=null;
	public Display display=null;
	public Shell shellPrincipal = null;
	public String titulo="PCuaderno";
	public String descripcion=mensajes.getString("Description");
	public String usando=mensajes.getString("Using");
	public String version="0.2";
	private TabFolder tabFolderClasificador = null;
	private TabItem tabItemClase; public final String tituloClase=mensajes.getString("Class"); 
	private TabItem tabItemHorario; public final String tituloHorario=mensajes.getString("ClassSchedule");
	private TabItem tabItemCalendario; public final String tituloCalendario=mensajes.getString("Calendar");
	private TabItem tabItemEventos; public final String tituloEventos=mensajes.getString("Events");
	private TabItem tabItemEvaluables; public final String tituloEvaluables=mensajes.getString("Assessable");
	private TabItem tabItemProgramacion; public final String tituloProgramacion=mensajes.getString("Programming");
	private TabItem tabItemMantenimiento; public final String tituloMantenimiento=mensajes.getString("Maintenance");
	private TabItem tabItemMatricula; public final String tituloMatricula=mensajes.getString("Enrollment");
	public GrupoClase grupoClase = null;
	public Horario horario = null;
	public Calendario calendario = null;
	public Evento evento = null;
	public Evaluable evaluable = null;
	public Programacion programacion = null;
	public Mantenimiento mantenimiento = null;
	public Matricula matricula = null;
	private Integer nivelNotificable=0;
	public static final boolean noti=false;

	public void notificar(String texto,boolean visible){
		String clase=Thread.currentThread().getStackTrace()[2].getClassName();
		String metodo=Thread.currentThread().getStackTrace()[2].getMethodName();
		String tabulador=((metodo.equals("widgetSelected"))?"--":"  ");
		String sangria="";for (int i=Thread.currentThread().getStackTrace().length;i>3;i--) sangria=sangria+tabulador;
		if (visible) nivelNotificable--;
		else nivelNotificable++;
		System.out.println(sangria+nivelNotificable+": "+clase+"."+metodo+": "+texto);
	}
	public void notificar(String texto){
		if (PCuaderno.MiPCuaderno.nivelNotificable.equals(0)) {
			String clase=Thread.currentThread().getStackTrace()[2].getClassName();
			String metodo=Thread.currentThread().getStackTrace()[2].getMethodName();
			//String traza="";
			String tabulador=((metodo.equals("widgetSelected"))?"--":"  ");
			String sangria="";for (int i=Thread.currentThread().getStackTrace().length;i>3;i--) {
				sangria=sangria+tabulador;
				//traza=traza+Thread.currentThread().getStackTrace()[i-1].getClassName()+"."+Thread.currentThread().getStackTrace()[i-1].getMethodName()+">";
			}
			System.out.println(sangria+clase+"."+metodo+": "+texto);//+"  FechaTrabajo="+Fecha.formaFecha(Fecha.fechaDeTrabajo));//+" |||"+traza);
		}
	}
	public void informacion(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_INFORMATION | SWT.OK);
		messageBox.setText(titulo+" - "+mensajes.getString("Information"));messageBox.setMessage(m);messageBox.open();
	}
	public int pregunta(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
		messageBox.setText(titulo+" - "+mensajes.getString("Question"));messageBox.setMessage(m);return messageBox.open();
	}
	public void error(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_ERROR | SWT.OK);
		messageBox.setText(titulo+" - "+mensajes.getString("Error"));messageBox.setMessage(m);messageBox.open();
	}
	public int aviso(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_WARNING | SWT.OK | SWT.CANCEL);
		messageBox.setText(titulo+" - "+mensajes.getString("Warning"));messageBox.setMessage(m);return messageBox.open();
	}
	public String entrada(String campo,String texto,boolean clave){
		Entrada entrada = new Entrada(PCuaderno.MiPCuaderno.shellPrincipal,campo,texto,clave);
		return entrada.open();
	}
	public String md5(String cadena){
		String md5Cadena=null;
		if (cadena!=null) {
			MessageDigest md = null;
			try {md = MessageDigest.getInstance("MD5");
			} catch (NoSuchAlgorithmException e) {e.printStackTrace();}
			md.reset();
	        byte[] md5Digest = md.digest(cadena.getBytes());
	        BigInteger md5Numero = new BigInteger(1, md5Digest);
	        md5Cadena = md5Numero.toString(16);
	        md5Cadena = (md5Cadena.length()==31)?"0"+md5Cadena:md5Cadena;
		}
		return md5Cadena;
	}
	private void createTabFolderPrincipal() {
		GridData gridDataClasificador = new GridData();
		gridDataClasificador.horizontalAlignment = GridData.FILL;
		gridDataClasificador.grabExcessHorizontalSpace = true;
		gridDataClasificador.grabExcessVerticalSpace = true;
		gridDataClasificador.verticalAlignment = GridData.FILL;
		tabFolderClasificador = new TabFolder(shellPrincipal, SWT.BORDER);
		tabFolderClasificador.setLayoutData(gridDataClasificador);
		tabItemClase = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemClase.setText(tituloClase);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making class...");//Parcial.imprimirParciales();
		createClase();
		tabItemClase.setControl(grupoClase);
		tabItemHorario = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemHorario.setText(tituloHorario);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making ClassSchedule...");//Parcial.imprimirParciales();
		createHorario();
		tabItemHorario.setControl(horario);
		tabItemCalendario = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemCalendario.setText(tituloCalendario);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making Calendar...");//Parcial.imprimirParciales();
		createCalendario();
		tabItemCalendario.setControl(calendario);
		tabItemEventos = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemEventos.setText(tituloEventos);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making Events...");//Parcial.imprimirParciales();
		createEvento();
		tabItemEventos.setControl(evento);
		tabItemEvaluables = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemEvaluables.setText(tituloEvaluables);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making Assessable...");//Parcial.imprimirParciales();
		createEvaluable();
		tabItemEvaluables.setControl(evaluable);
		tabItemProgramacion = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemProgramacion.setText(tituloProgramacion);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making Programming...");//Parcial.imprimirParciales();
		createProgramacion();
		tabItemProgramacion.setControl(programacion);
		tabItemMantenimiento = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemMantenimiento.setText(tituloMantenimiento);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making Maintenance...");//Parcial.imprimirParciales();
		createMantener();
		tabItemMantenimiento.setControl(mantenimiento);
		tabItemMatricula = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemMatricula.setText(tituloMatricula);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making Enrollment...");//Parcial.imprimirParciales();
		createMatricula();
		tabItemMatricula.setControl(matricula);
		tabFolderClasificador
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("tabFolderClassifier");
						//if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("widgetSelected()"+((TabItem) e.item).getText());
						if (((TabItem) e.item).getText().equals(tituloClase)){
							grupoClase.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloHorario)){
							horario.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloCalendario)){
							calendario.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloEventos)){
							evento.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloEvaluables)){
							evaluable.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloProgramacion)){
							programacion.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloMantenimiento)){
							mantenimiento.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloMatricula)){
							matricula.actualizar();
						}
					}
				});
	}
	public void createClase(){grupoClase = new GrupoClase(tabFolderClasificador,SWT.NONE);}
	public void createHorario(){horario = new Horario(tabFolderClasificador,SWT.NONE);}
	public void createCalendario(){calendario = new Calendario(tabFolderClasificador,SWT.NONE);}
	public void createEvento(){evento = new Evento(tabFolderClasificador,SWT.NONE);}
	public void createEvaluable(){evaluable = new Evaluable(tabFolderClasificador,SWT.NONE);}
	public void createProgramacion(){programacion = new Programacion(tabFolderClasificador,SWT.NONE);}
	public void createMantener(){mantenimiento = new Mantenimiento(tabFolderClasificador,SWT.NONE);}
	public void createMatricula(){matricula = new Matricula(tabFolderClasificador,SWT.NONE);}
	public void iniciar(){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Starting configuration...");
		BBDD.actualizaConfiguracion();
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Starting dates...");
		Fecha.inicializar();
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Starting SubjectClass...");
		MateriaClase.inicializar();
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Starting Partial...");
		Parcial.inicializar();
	}

	public static void main(String[] args) {
		System.out.println("Starting application...");
		System.out.println("Making class PCuaderno...");
		PCuaderno thisClass = new PCuaderno();
		PCuaderno.MiPCuaderno=thisClass;
		thisClass.display = Display.getDefault();
		thisClass.createSShellPricipal();
		boolean vacio=!(new File(BBDD.archivoSCRIPT)).exists();
		File viejo = null;
		if (vacio) {  // In version 0.2 change folder database name to PCuaderno_bbdd
			viejo = new File(BBDD.dirViejoBBDD);
			if (viejo.exists()) {
				System.out.print("Update folder database name ... ");
				File nuevo = new File(BBDD.dirBBDD);
				if (viejo.renameTo(nuevo)) {
					System.out.println("Updated");
					vacio=!(new File(BBDD.archivoSCRIPT)).exists();
				} else
					System.out.println("Error - Can't rename folder");
			}
		}
		System.out.println("Asking encryption key...");
		String m=((vacio)?thisClass.mensajes.getString("EmptyInit")
				:thisClass.mensajes.getString("Init1")+" '"+PCuaderno.MiPCuaderno.titulo+"_bbdd"+"' "+thisClass.mensajes.getString("Init2"));
		m+="\n\n"+thisClass.mensajes.getString("Licencia");
		String acceso=PCuaderno.MiPCuaderno.md5(PCuaderno.MiPCuaderno.entrada(thisClass.mensajes.getString("EncriptionKey"),m,true));
		if (acceso!=null) {
			if (BBDD.acceso(acceso)){
				acceso=null;
				System.out.println(thisClass.mensajes.getString("RemovingLock"));
				BBDD.borrarArchivo(BBDD.archivoLCK);
				if (vacio) {BBDD.crear();BBDD.valoresIniciales();BBDD.valoresConfiguracion();}
				thisClass.iniciar();
				thisClass.createTabFolderPrincipal();
				Monitor primary = thisClass.display.getPrimaryMonitor ();
				Rectangle bounds = primary.getBounds ();
				Rectangle rect = thisClass.shellPrincipal.getBounds ();
				int x = bounds.x + (bounds.width - rect.width) / 2;
				int y = bounds.y + (bounds.height - rect.height) / 2;
				thisClass.shellPrincipal.setLocation (x, y);
				thisClass.shellPrincipal.open();
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Ready -----------------------");
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("-----------------------------");
				while (!thisClass.shellPrincipal.isDisposed()) {
					if (!thisClass.display.readAndDispatch()) {
						thisClass.display.sleep();
						thisClass.display.readAndDispatch();
					}
				}
				TextoDemorado.escribeSiHay();
				BBDD.cerrarBBDD();
			} else {
				m=thisClass.mensajes.getString("NoConnectBBDD");
				PCuaderno.MiPCuaderno.informacion(m);
			}
			acceso=null;
		}
		thisClass.shellPrincipal.dispose();
		thisClass.display.dispose();
	}
	private void createSShellPricipal() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 2;
		gridLayout.marginWidth = 2;
		gridLayout.marginHeight = 2;
		gridLayout.verticalSpacing = 2;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		shellPrincipal = new Shell(display);
		shellPrincipal.setText(titulo);
		shellPrincipal.setLayout(gridLayout);
		shellPrincipal.setSize(new Point(700, 400));
	}
	public void seleccionarFicha(String titulo){
		if (titulo.equals(tituloClase))
			tabFolderClasificador.setSelection(tabItemClase);
		else if (titulo.equals(tituloHorario))
			tabFolderClasificador.setSelection(tabItemHorario);
		else if (titulo.equals(tituloCalendario))
			tabFolderClasificador.setSelection(tabItemCalendario);
		else if (titulo.equals(tituloEventos))
			tabFolderClasificador.setSelection(tabItemEventos);
		else if (titulo.equals(tituloEvaluables))
			tabFolderClasificador.setSelection(tabItemEvaluables);
		else if (titulo.equals(tituloProgramacion))
			tabFolderClasificador.setSelection(tabItemProgramacion);
		else if (titulo.equals(tituloMantenimiento))
			tabFolderClasificador.setSelection(tabItemMantenimiento);
		else if (titulo.equals(tituloMatricula))
			tabFolderClasificador.setSelection(tabItemMatricula);
	}
}
